package com.example.ardubotrc_control;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.os.Handler;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;
import android.view.View;
import android.widget.FrameLayout;

public class GameSurface extends SurfaceView implements SurfaceHolder.Callback {

	private Context _context;
	private GameThread _thread;
	private GameControls _controls;

	private GameJoystick _joystick;


	private Bitmap _pointer;
	
	private double radius_old = 0, angle_old = 0;

	public GameSurface(Context context) {
		super(context);
		// TODO Auto-generated constructor stub
		_context = context;
		init();
		
	}

	private void init(){
		//initialize our screen holder
		SurfaceHolder holder = getHolder();
		holder.addCallback( this);

		//initialize our game engine

		//initialize our Thread class. A call will be made to start it later
		_thread = new GameThread(holder, _context, new Handler(),this);
		setFocusable(true);


		_joystick = new GameJoystick(getContext().getResources());
		//_pointer = (Bitmap)BitmapFactory.decodeResource(getResources(), R.drawable.icon);
		//controls
		_controls = new GameControls();
		setOnTouchListener(_controls);
	}


	public void doDraw(Canvas canvas){
		// get location of surfaceview for touch-offset
		int[] loc = new int[2];
		this.getLocationOnScreen(loc);
		_controls.touchOffset.x = loc[0];
		_controls.touchOffset.y = loc[1];
		
		_controls.update(null);
		//canvas.setDensity(72);
		
		//dynamically set control position from canvas size
		_controls._controlPosition.x = canvas.getWidth() - _joystick.get_joystickBg().getWidth() - 40;
		_controls._controlPosition.y = canvas.getHeight() - _joystick.get_joystickBg().getHeight() - 40;
		//dynamically calculate joystick origin from control position
		_controls._joystickOrigin.x = _controls._controlPosition.x + _joystick.get_joystickBg().getWidth()/2;
		_controls._joystickOrigin.y = _controls._controlPosition.y + _joystick.get_joystickBg().getHeight()/2;
		
		
		//get the angle
		double angle = Math.atan2(_controls._touchingPoint.y - (float)_controls._joystickOrigin.y,_controls._touchingPoint.x - (float)_controls._joystickOrigin.x)/(Math.PI/180);
		double radius = Math.sqrt(Math.pow(((_controls._touchingPoint.x - (float)_controls._joystickOrigin.x)/_joystick.get_joystickBg().getWidth()*2),2) + Math.pow(((_controls._touchingPoint.y - (float)_controls._joystickOrigin.y)/_joystick.get_joystickBg().getHeight()*2),2));
		
		// limit the pointer position to the background circle
		if (radius > 1){
			_controls._touchingPoint.x = (int)(_controls._joystickOrigin.x + _joystick.get_joystickBg().getWidth()/2 * Math.cos(angle*Math.PI/180));
			_controls._touchingPoint.y = (int)(_controls._joystickOrigin.y + _joystick.get_joystickBg().getHeight()/2 * Math.sin(angle*Math.PI/180));
			radius = 1;
		}
		angle+=90;
		if (angle<0)
			angle+=360;
		
		
		//draw the joystick background
		canvas.drawBitmap(_joystick.get_joystickBg(), _controls._controlPosition.x, _controls._controlPosition.y, null);

		//draw the dragable joystick
		canvas.drawBitmap(_joystick.get_joystick(),_controls._touchingPoint.x - _joystick.get_joystick().getWidth()/2, _controls._touchingPoint.y - _joystick.get_joystick().getHeight()/2, null);
		//Log.d("Canvas-Size",Integer.toString(canvas.getWidth()) +" x " +Integer.toString(canvas.getHeight()));
		//Log.d("Joystick-Size",Integer.toString(_joystick.get_joystick().getWidth()) +" x " +Integer.toString(_joystick.get_joystick().getHeight()));
		//Log.d("JoystickBG-Size",Integer.toString(_joystick.get_joystickBg().getWidth()) +" x " +Integer.toString(_joystick.get_joystickBg().getHeight()));
		//Log.d("Angle : Radius",Double.toString(angle) +" x " +Double.toString(radius));
		//Log.d("surfaceview",Integer.toString(loc[1]));
		int rad = (int)(radius*100);
		int ang = (int)(angle);
		
		Log.d("send",Integer.toString(ang) + " " + Integer.toString(rad));
		
		if ((radius != radius_old) || (angle != angle_old))
			MainActivity.sendString("a" + Integer.toString(ang) + "r" + Integer.toString(rad) + "x");
		radius_old = radius;
		angle_old = angle;
	}



	//these methods are overridden from the SurfaceView super class. They are automatically called 
	//when a SurfaceView is created, resumed or suspended.
	@Override 
	public void surfaceChanged(SurfaceHolder arg0, int arg1, int arg2, int arg3) {}
	private boolean retry;
	@Override 
	public void surfaceDestroyed(SurfaceHolder arg0) {
		retry = true;
		//code to end gameloop
		_thread.state = GameThread.STOPED;
		while (retry) {
			try {
				//code to kill Thread
				_thread.join();
				retry = false;
			} catch (InterruptedException e) {
			}
		}

	}

	@Override 
	public void surfaceCreated(SurfaceHolder arg0) {
		if(_thread.state==GameThread.PAUSED){
			//When game is opened again in the Android OS
			_thread = new GameThread(getHolder(), _context, new Handler(),this);
			_thread.start();
		}else{
			//creating the game Thread for the first time
			_thread.start();
		}
	}

	public void Update() {
		// TODO Auto-generated method stub

	}



}
