package com.example.ardubotrc_control;

import java.io.IOException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import android.app.Activity;
import android.content.Context;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.TextView;

import com.hoho.android.usbserial.R;
import com.hoho.android.usbserial.driver.UsbSerialDriver;
import com.hoho.android.usbserial.driver.UsbSerialProber;
import com.hoho.android.usbserial.util.HexDump;
import com.hoho.android.usbserial.util.SerialInputOutputManager;

public class MainActivity extends Activity {
	
	/**
     * The device currently in use, or {@code null}.
     */
    private UsbSerialDriver mSerialDevice;

    /**
     * The system's USB service.
     */
    private UsbManager mUsbManager;
    
    private TextView mTitleTextView;
    private TextView mDebugTextView;
    
    private final ExecutorService mExecutor = Executors.newSingleThreadExecutor();

    public static SerialInputOutputManager mSerialIoManager;

    private final SerialInputOutputManager.Listener mListener =
            new SerialInputOutputManager.Listener() {

        @Override
        public void onRunError(Exception e) {
            Log.d("USB-Serial", "Runner stopped.");
            mDebugTextView.append("DEBUG: Runner stopped.\n");

        }

        @Override
        public void onNewData(final byte[] data) {
            MainActivity.this.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    MainActivity.this.updateReceivedData(data);
                }
            });
        }
    };

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
        
        //set to full screen 
      	this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,   
      			WindowManager.LayoutParams.FLAG_FULLSCREEN);
      	this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        
        //setContentView(new GameSurface(this));
      	setContentView(R.layout.activity_main);
        
        mUsbManager = (UsbManager) getSystemService(Context.USB_SERVICE);
        
        mTitleTextView = (TextView) findViewById(R.id.demoTitle);
        mDebugTextView = (TextView) findViewById(R.id.debugText);
        
        ((Button)findViewById(R.id.buttonConnect)).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mSerialIoManager!=null)
                mSerialIoManager.writeAsync("acx".getBytes());
                //mSerialIoManager.writeAsync("a0r0x".getBytes());
            }
        });
        
        
        FrameLayout frame = (FrameLayout) findViewById(R.id.frameLayout1);
        GameSurface gamesurface = new GameSurface(this);
        frame.addView(gamesurface);
    }
    
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }
    
    
    @Override
    protected void onPause() {
        super.onPause();
        stopIoManager();
        if (mSerialDevice != null) {
            try {
                mSerialDevice.close();
            } catch (IOException e) {
                // Ignore.
            }
            mSerialDevice = null;
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        mSerialDevice = UsbSerialProber.acquire(mUsbManager);
        Log.d("USB-Serial", "Resumed, mSerialDevice=" + mSerialDevice);
        mDebugTextView.append("DEBUG: Resumed, mSerialDevice=" + mSerialDevice + "\n");
        if (mSerialDevice == null) {
            mTitleTextView.setText("No serial device.");
        } else {
            try {
                mSerialDevice.open();
            } catch (IOException e) {
                Log.e("USB-Serial", "Error setting up device: " + e.getMessage(), e);
                mTitleTextView.setText("Error opening device: " + e.getMessage());
                try {
                    mSerialDevice.close();
                } catch (IOException e2) {
                    // Ignore.
                }
                mSerialDevice = null;
                return;
            }
            mTitleTextView.setText("Serial device: " + mSerialDevice);
        }
        onDeviceStateChange();
    }

    private void stopIoManager() {
        if (mSerialIoManager != null) {
            Log.i("USB-Serial", "Stopping io manager ..");
            mDebugTextView.append("DEBUG: Stopping io manager .. \n");

            mSerialIoManager.stop();
            mSerialIoManager = null;
        }
    }

    private void startIoManager() {
        if (mSerialDevice != null) {
            Log.i("USB-Serial", "Starting io manager ..");
            mDebugTextView.append("DEBUG: Starting io manager ..\n");
            mSerialIoManager = new SerialInputOutputManager(mSerialDevice, mListener);
            mExecutor.submit(mSerialIoManager);
        }
    }

    private void onDeviceStateChange() {
        stopIoManager();
        startIoManager();
    }

    private void updateReceivedData(byte[] data) {
        final String message = "Read " + data.length + " bytes: \n"
                + HexDump.dumpHexString(data) + "\n\n";
        //mDumpTextView.append(message);
        //mScrollView.smoothScrollTo(0, mDumpTextView.getBottom());
    }
    
    public static void sendString(String string){
    	if (mSerialIoManager!=null)
            mSerialIoManager.writeAsync(string.getBytes());
    }
}
